@@ -1,15 +1,15 @@
-
 #if __WORDSIZE == 64
-#define JIT_INSTR_MAX 76
+#define JIT_INSTR_MAX 168
     0,	/* data */
     0,	/* live */
     4,	/* align */
     0,	/* save */
     0,	/* load */
+    0,  /* skip */
     0,	/* #name */
     0,	/* #note */
     0,	/* label */
-    76,	/* prolog */
+    88,	/* prolog */
     0,	/* ellipsis */
     0,	/* va_push */
     0,	/* allocai */
@@ -24,9 +24,9 @@
     0,	/* getarg_l */
     0,	/* putargr */
     0,	/* putargi */
-    0,	/* va_start */
-    0,	/* va_arg */
-    0,	/* va_arg_d */
+    20,	/* va_start */
+    24,	/* va_arg */
+    44,	/* va_arg_d */
     0,	/* va_end */
     4,	/* addr */
     32,	/* addi */
@@ -47,18 +47,18 @@
     56,	/* qmuli */
     12,	/* qmulr_u */
     32,	/* qmuli_u */
-    48,	/* divr */
-    72,	/* divi */
-    48,	/* divr_u */
-    72,	/* divi_u */
-    56,	/* qdivr */
-    56,	/* qdivi */
-    56,	/* qdivr_u */
-    56,	/* qdivi_u */
-    48,	/* remr */
-    72,	/* remi */
-    48,	/* remr_u */
-    72,	/* remi_u */
+    44,	/* divr */
+    68,	/* divi */
+    44,	/* divr_u */
+    68,	/* divi_u */
+    52,	/* qdivr */
+    52,	/* qdivi */
+    52,	/* qdivr_u */
+    52,	/* qdivi_u */
+    44,	/* remr */
+    68,	/* remi */
+    44,	/* remr_u */
+    68,	/* remi_u */
     4,	/* andr */
     32,	/* andi */
     4,	/* orr */
@@ -95,6 +95,8 @@
     12,	/* nei */
     4,	/* movr */
     32,	/* movi */
+    4,	/* movnr */
+    4,	/* movzr */
     8,	/* extr_c */
     8,	/* extr_uc */
     8,	/* extr_s */
@@ -119,19 +121,19 @@
     4,	/* ldr_l */
     32,	/* ldi_l */
     16,	/* ldxr_c */
-    12,	/* ldxi_c */
+    44,	/* ldxi_c */
     8,	/* ldxr_uc */
-    4,	/* ldxi_uc */
+    36,	/* ldxi_uc */
     16,	/* ldxr_s */
-    12,	/* ldxi_s */
+    44,	/* ldxi_s */
     8,	/* ldxr_us */
-    4,	/* ldxi_us */
+    36,	/* ldxi_us */
     8,	/* ldxr_i */
-    4,	/* ldxi_i */
+    36,	/* ldxi_i */
     16,	/* ldxr_ui */
-    12,	/* ldxi_ui */
+    44,	/* ldxi_ui */
     8,	/* ldxr_l */
-    4,	/* ldxi_l */
+    36,	/* ldxi_l */
     4,	/* str_c */
     32,	/* sti_c */
     4,	/* str_s */
@@ -141,13 +143,13 @@
     4,	/* str_l */
     32,	/* sti_l */
     8,	/* stxr_c */
-    4,	/* stxi_c */
+    36,	/* stxi_c */
     8,	/* stxr_s */
-    4,	/* stxi_s */
+    36,	/* stxi_s */
     8,	/* stxr_i */
-    4,	/* stxi_i */
+    36,	/* stxi_i */
     8,	/* stxr_l */
-    4,	/* stxi_l */
+    36,	/* stxi_l */
     8,	/* bltr */
     8,	/* blti */
     8,	/* bltr_u */
@@ -188,7 +190,7 @@
     32,	/* bxsubi */
     16,	/* bxsubr_u */
     16,	/* bxsubi_u */
-    0,	/* jmpr */
+    4,	/* jmpr */
     36,	/* jmpi */
     8,	/* callr */
     36,	/* calli */
@@ -207,93 +209,93 @@
     0,	/* retval_i */
     0,	/* retval_ui */
     0,	/* retval_l */
-    68,	/* epilog */
+    76,	/* epilog */
     0,	/* arg_f */
     0,	/* getarg_f */
     0,	/* putargr_f */
     0,	/* putargi_f */
     8,	/* addr_f */
-    32,	/* addi_f */
+    40,	/* addi_f */
     8,	/* subr_f */
-    32,	/* subi_f */
-    32,	/* rsbi_f */
+    40,	/* subi_f */
+    40,	/* rsbi_f */
     8,	/* mulr_f */
-    32,	/* muli_f */
+    40,	/* muli_f */
     8,	/* divr_f */
-    32,	/* divi_f */
+    40,	/* divi_f */
     4,	/* negr_f */
     4,	/* absr_f */
     8,	/* sqrtr_f */
     32,	/* ltr_f */
-    56,	/* lti_f */
+    64,	/* lti_f */
     32,	/* ler_f */
-    56,	/* lei_f */
+    64,	/* lei_f */
     32,	/* eqr_f */
-    56,	/* eqi_f */
+    64,	/* eqi_f */
     32,	/* ger_f */
-    56,	/* gei_f */
+    64,	/* gei_f */
     32,	/* gtr_f */
-    56,	/* gti_f */
+    64,	/* gti_f */
     32,	/* ner_f */
-    56,	/* nei_f */
+    64,	/* nei_f */
     32,	/* unltr_f */
-    56,	/* unlti_f */
+    64,	/* unlti_f */
     32,	/* unler_f */
-    56,	/* unlei_f */
+    64,	/* unlei_f */
     32,	/* uneqr_f */
-    56,	/* uneqi_f */
+    64,	/* uneqi_f */
     32,	/* unger_f */
-    56,	/* ungei_f */
+    64,	/* ungei_f */
     32,	/* ungtr_f */
-    56,	/* ungti_f */
+    64,	/* ungti_f */
     32,	/* ltgtr_f */
-    56,	/* ltgti_f */
+    64,	/* ltgti_f */
     20,	/* ordr_f */
-    44,	/* ordi_f */
+    52,	/* ordi_f */
     20,	/* unordr_f */
-    44,	/* unordi_f */
+    52,	/* unordi_f */
     16,	/* truncr_f_i */
     16,	/* truncr_f_l */
     12,	/* extr_f */
     4,	/* extr_d_f */
     4,	/* movr_f */
-    24,	/* movi_f */
+    32,	/* movi_f */
     4,	/* ldr_f */
     32,	/* ldi_f */
     8,	/* ldxr_f */
-    4,	/* ldxi_f */
+    36,	/* ldxi_f */
     4,	/* str_f */
     32,	/* sti_f */
     8,	/* stxr_f */
-    4,	/* stxi_f */
+    36,	/* stxi_f */
     24,	/* bltr_f */
-    48,	/* blti_f */
+    56,	/* blti_f */
     24,	/* bler_f */
-    48,	/* blei_f */
+    56,	/* blei_f */
     24,	/* beqr_f */
-    48,	/* beqi_f */
+    56,	/* beqi_f */
     24,	/* bger_f */
-    48,	/* bgei_f */
+    56,	/* bgei_f */
     24,	/* bgtr_f */
-    48,	/* bgti_f */
+    56,	/* bgti_f */
     28,	/* bner_f */
-    52,	/* bnei_f */
+    60,	/* bnei_f */
     28,	/* bunltr_f */
-    52,	/* bunlti_f */
+    60,	/* bunlti_f */
     28,	/* bunler_f */
-    52,	/* bunlei_f */
+    60,	/* bunlei_f */
     28,	/* buneqr_f */
-    52,	/* buneqi_f */
+    60,	/* buneqi_f */
     28,	/* bunger_f */
-    52,	/* bungei_f */
+    60,	/* bungei_f */
     28,	/* bungtr_f */
-    52,	/* bungti_f */
+    60,	/* bungti_f */
     28,	/* bltgtr_f */
-    52,	/* bltgti_f */
+    60,	/* bltgti_f */
     12,	/* bordr_f */
-    36,	/* bordi_f */
+    44,	/* bordi_f */
     12,	/* bunordr_f */
-    36,	/* bunordi_f */
+    44,	/* bunordi_f */
     0,	/* pushargr_f */
     0,	/* pushargi_f */
     0,	/* retr_f */
@@ -304,87 +306,87 @@
     0,	/* putargr_d */
     0,	/* putargi_d */
     8,	/* addr_d */
-    28,	/* addi_d */
+    40,	/* addi_d */
     8,	/* subr_d */
-    28,	/* subi_d */
-    28,	/* rsbi_d */
+    40,	/* subi_d */
+    40,	/* rsbi_d */
     8,	/* mulr_d */
-    28,	/* muli_d */
+    40,	/* muli_d */
     8,	/* divr_d */
-    28,	/* divi_d */
+    40,	/* divi_d */
     4,	/* negr_d */
     4,	/* absr_d */
     8,	/* sqrtr_d */
     32,	/* ltr_d */
-    52,	/* lti_d */
+    64,	/* lti_d */
     32,	/* ler_d */
-    52,	/* lei_d */
+    64,	/* lei_d */
     32,	/* eqr_d */
-    52,	/* eqi_d */
+    64,	/* eqi_d */
     32,	/* ger_d */
-    52,	/* gei_d */
+    64,	/* gei_d */
     32,	/* gtr_d */
-    52,	/* gti_d */
+    64,	/* gti_d */
     32,	/* ner_d */
-    52,	/* nei_d */
+    64,	/* nei_d */
     32,	/* unltr_d */
-    52,	/* unlti_d */
+    64,	/* unlti_d */
     32,	/* unler_d */
-    52,	/* unlei_d */
+    64,	/* unlei_d */
     32,	/* uneqr_d */
-    52,	/* uneqi_d */
+    64,	/* uneqi_d */
     32,	/* unger_d */
-    52,	/* ungei_d */
+    64,	/* ungei_d */
     32,	/* ungtr_d */
-    52,	/* ungti_d */
+    64,	/* ungti_d */
     32,	/* ltgtr_d */
-    52,	/* ltgti_d */
+    64,	/* ltgti_d */
     20,	/* ordr_d */
-    40,	/* ordi_d */
+    52,	/* ordi_d */
     20,	/* unordr_d */
-    40,	/* unordi_d */
+    52,	/* unordi_d */
     16,	/* truncr_d_i */
     16,	/* truncr_d_l */
     12,	/* extr_d */
     4,	/* extr_f_d */
     4,	/* movr_d */
-    20,	/* movi_d */
+    32,	/* movi_d */
     4,	/* ldr_d */
     32,	/* ldi_d */
     8,	/* ldxr_d */
-    4,	/* ldxi_d */
+    36,	/* ldxi_d */
     4,	/* str_d */
     32,	/* sti_d */
     8,	/* stxr_d */
-    4,	/* stxi_d */
+    36,	/* stxi_d */
     24,	/* bltr_d */
-    44,	/* blti_d */
+    56,	/* blti_d */
     24,	/* bler_d */
-    44,	/* blei_d */
+    56,	/* blei_d */
     24,	/* beqr_d */
-    44,	/* beqi_d */
+    56,	/* beqi_d */
     24,	/* bger_d */
-    44,	/* bgei_d */
+    56,	/* bgei_d */
     24,	/* bgtr_d */
-    44,	/* bgti_d */
+    56,	/* bgti_d */
     28,	/* bner_d */
-    48,	/* bnei_d */
+    60,	/* bnei_d */
     28,	/* bunltr_d */
-    48,	/* bunlti_d */
+    60,	/* bunlti_d */
     28,	/* bunler_d */
-    48,	/* bunlei_d */
+    60,	/* bunlei_d */
     28,	/* buneqr_d */
-    48,	/* buneqi_d */
+    60,	/* buneqi_d */
     28,	/* bunger_d */
-    48,	/* bungei_d */
+    60,	/* bungei_d */
     28,	/* bungtr_d */
-    48,	/* bungti_d */
+    60,	/* bungti_d */
     28,	/* bltgtr_d */
-    48,	/* bltgti_d */
+    60,	/* bltgti_d */
     12,	/* bordr_d */
-    32,	/* bordi_d */
+    44,	/* bordi_d */
     12,	/* bunordr_d */
-    32,	/* bunordi_d */
+    44,	/* bunordi_d */
     0,	/* pushargr_d */
     0,	/* pushargi_d */
     0,	/* retr_d */
@@ -399,4 +401,9 @@
     0,	/* movi_d_ww */
     0,	/* movr_d_w */
     0,	/* movi_d_w */
+    16,	/* bswapr_us */
+    36,	/* bswapr_ui */
+    36,	/* bswapr_ul */
+    32,	/* casr */
+    60,	/* casi */
 #endif /* __WORDSIZE */
