@@ -1,15 +1,16 @@
 
 #if __WORDSIZE == 32
-#define JIT_INSTR_MAX 64
+#define JIT_INSTR_MAX 196
     0,	/* data */
     0,	/* live */
     0,	/* align */
     0,	/* save */
     0,	/* load */
+    0,  /* skip */
     0,	/* #name */
     0,	/* #note */
     0,	/* label */
-    64,	/* prolog */
+    156,	/* prolog */
     0,	/* ellipsis */
     0,	/* va_push */
     0,	/* allocai */
@@ -24,9 +25,9 @@
     0,	/* getarg_l */
     0,	/* putargr */
     0,	/* putargi */
-    0,	/* va_start */
-    0,	/* va_arg */
-    0,	/* va_arg_d */
+    4,	/* va_start */
+    8,	/* va_arg */
+    20,	/* va_arg_d */
     0,	/* va_end */
     4,	/* addr */
     12,	/* addi */
@@ -40,13 +41,13 @@
     12,	/* subci */
     4,	/* subxr */
     8,	/* subxi */
-    16,	/* rsbi */
-    28,	/* mulr */
-    36,	/* muli */
+    12,	/* rsbi */
+    48,	/* mulr */
+    56,	/* muli */
     40,	/* qmulr */
     44,	/* qmuli */
-    32,	/* qmulr_u */
-    40,	/* qmuli_u */
+    52,	/* qmulr_u */
+    60,	/* qmuli_u */
     36,	/* divr */
     40,	/* divi */
     36,	/* divr_u */
@@ -95,6 +96,8 @@
     8,	/* nei */
     4,	/* movr */
     8,	/* movi */
+    12,	/* movnr */
+    12,	/* movzr */
     4,	/* extr_c */
     4,	/* extr_uc */
     4,	/* extr_s */
@@ -103,7 +106,7 @@
     0,	/* extr_ui */
     4,	/* htonr_us */
     4,	/* htonr_ui */
-    0,	/* htonr_l */
+    0,	/* htonr_ul */
     8,	/* ldr_c */
     12,	/* ldi_c */
     4,	/* ldr_uc */
@@ -119,15 +122,15 @@
     0,	/* ldr_l */
     0,	/* ldi_l */
     8,	/* ldxr_c */
-    8,	/* ldxi_c */
+    12,	/* ldxi_c */
     4,	/* ldxr_uc */
-    4,	/* ldxi_uc */
+    8,	/* ldxi_uc */
     8,	/* ldxr_s */
-    8,	/* ldxi_s */
+    12,	/* ldxi_s */
     4,	/* ldxr_us */
-    4,	/* ldxi_us */
+    8,	/* ldxi_us */
     4,	/* ldxr_i */
-    4,	/* ldxi_i */
+    8,	/* ldxi_i */
     0,	/* ldxr_ui */
     0,	/* ldxi_ui */
     0,	/* ldxr_l */
@@ -141,11 +144,11 @@
     0,	/* str_l */
     0,	/* sti_l */
     8,	/* stxr_c */
-    4,	/* stxi_c */
+    12,	/* stxi_c */
     8,	/* stxr_s */
-    4,	/* stxi_s */
+    12,	/* stxi_s */
     8,	/* stxr_i */
-    4,	/* stxi_i */
+    12,	/* stxi_i */
     0,	/* stxr_l */
     0,	/* stxi_l */
     8,	/* bltr */
@@ -188,7 +191,7 @@
     16,	/* bxsubi */
     16,	/* bxsubr_u */
     20,	/* bxsubi_u */
-    0,	/* jmpr */
+    4,	/* jmpr */
     12,	/* jmpi */
     40,	/* callr */
     44,	/* calli */
@@ -207,93 +210,93 @@
     0,	/* retval_i */
     0,	/* retval_ui */
     0,	/* retval_l */
-    64,	/* epilog */
+    196,	/* epilog */
     0,	/* arg_f */
     0,	/* getarg_f */
     0,	/* putargr_f */
     0,	/* putargi_f */
     4,	/* addr_f */
-    16,	/* addi_f */
+    24,	/* addi_f */
     4,	/* subr_f */
-    16,	/* subi_f */
-    16,	/* rsbi_f */
+    24,	/* subi_f */
+    24,	/* rsbi_f */
     4,	/* mulr_f */
-    16,	/* muli_f */
+    24,	/* muli_f */
     4,	/* divr_f */
-    16,	/* divi_f */
+    24,	/* divi_f */
     4,	/* negr_f */
     4,	/* absr_f */
     4,	/* sqrtr_f */
     16,	/* ltr_f */
-    28,	/* lti_f */
+    36,	/* lti_f */
     16,	/* ler_f */
-    28,	/* lei_f */
+    36,	/* lei_f */
     16,	/* eqr_f */
-    28,	/* eqi_f */
+    36,	/* eqi_f */
     16,	/* ger_f */
-    28,	/* gei_f */
+    36,	/* gei_f */
     16,	/* gtr_f */
-    28,	/* gti_f */
+    36,	/* gti_f */
     16,	/* ner_f */
-    28,	/* nei_f */
+    36,	/* nei_f */
     16,	/* unltr_f */
-    28,	/* unlti_f */
+    36,	/* unlti_f */
     16,	/* unler_f */
-    28,	/* unlei_f */
+    36,	/* unlei_f */
     16,	/* uneqr_f */
-    28,	/* uneqi_f */
+    36,	/* uneqi_f */
     16,	/* unger_f */
-    28,	/* ungei_f */
+    36,	/* ungei_f */
     16,	/* ungtr_f */
-    28,	/* ungti_f */
+    36,	/* ungti_f */
     16,	/* ltgtr_f */
-    28,	/* ltgti_f */
+    36,	/* ltgti_f */
     16,	/* ordr_f */
-    28,	/* ordi_f */
+    36,	/* ordi_f */
     16,	/* unordr_f */
-    28,	/* unordi_f */
-    12,	/* truncr_f_i */
+    36,	/* unordi_f */
+    16,	/* truncr_f_i */
     0,	/* truncr_f_l */
-    12,	/* extr_f */
+    20,	/* extr_f */
     4,	/* extr_d_f */
     4,	/* movr_f */
-    12,	/* movi_f */
+    20,	/* movi_f */
     4,	/* ldr_f */
     12,	/* ldi_f */
-    4,	/* ldxr_f */
-    4,	/* ldxi_f */
+    8,	/* ldxr_f */
+    12,	/* ldxi_f */
     4,	/* str_f */
     12,	/* sti_f */
     8,	/* stxr_f */
-    4,	/* stxi_f */
+    12,	/* stxi_f */
     16,	/* bltr_f */
-    28,	/* blti_f */
+    36,	/* blti_f */
     16,	/* bler_f */
-    28,	/* blei_f */
+    36,	/* blei_f */
     16,	/* beqr_f */
-    28,	/* beqi_f */
+    36,	/* beqi_f */
     16,	/* bger_f */
-    28,	/* bgei_f */
+    36,	/* bgei_f */
     16,	/* bgtr_f */
-    28,	/* bgti_f */
+    36,	/* bgti_f */
     16,	/* bner_f */
-    28,	/* bnei_f */
+    36,	/* bnei_f */
     16,	/* bunltr_f */
-    28,	/* bunlti_f */
+    36,	/* bunlti_f */
     16,	/* bunler_f */
-    28,	/* bunlei_f */
+    36,	/* bunlei_f */
     16,	/* buneqr_f */
-    28,	/* buneqi_f */
+    36,	/* buneqi_f */
     16,	/* bunger_f */
-    28,	/* bungei_f */
+    36,	/* bungei_f */
     16,	/* bungtr_f */
-    28,	/* bungti_f */
+    36,	/* bungti_f */
     16,	/* bltgtr_f */
-    28,	/* bltgti_f */
+    36,	/* bltgti_f */
     16,	/* bordr_f */
-    28,	/* bordi_f */
+    36,	/* bordi_f */
     16,	/* bunordr_f */
-    28,	/* bunordi_f */
+    36,	/* bunordi_f */
     0,	/* pushargr_f */
     0,	/* pushargi_f */
     0,	/* retr_f */
@@ -304,87 +307,87 @@
     0,	/* putargr_d */
     0,	/* putargi_d */
     4,	/* addr_d */
-    24,	/* addi_d */
+    32,	/* addi_d */
     4,	/* subr_d */
-    24,	/* subi_d */
-    24,	/* rsbi_d */
+    32,	/* subi_d */
+    32,	/* rsbi_d */
     4,	/* mulr_d */
-    24,	/* muli_d */
+    32,	/* muli_d */
     4,	/* divr_d */
-    24,	/* divi_d */
+    32,	/* divi_d */
     4,	/* negr_d */
     4,	/* absr_d */
     4,	/* sqrtr_d */
     16,	/* ltr_d */
-    36,	/* lti_d */
+    44,	/* lti_d */
     16,	/* ler_d */
-    36,	/* lei_d */
+    44,	/* lei_d */
     16,	/* eqr_d */
-    36,	/* eqi_d */
+    44,	/* eqi_d */
     16,	/* ger_d */
-    36,	/* gei_d */
+    44,	/* gei_d */
     16,	/* gtr_d */
-    36,	/* gti_d */
+    44,	/* gti_d */
     16,	/* ner_d */
-    36,	/* nei_d */
+    44,	/* nei_d */
     16,	/* unltr_d */
-    36,	/* unlti_d */
+    44,	/* unlti_d */
     16,	/* unler_d */
-    36,	/* unlei_d */
+    44,	/* unlei_d */
     16,	/* uneqr_d */
-    36,	/* uneqi_d */
+    44,	/* uneqi_d */
     16,	/* unger_d */
-    36,	/* ungei_d */
+    44,	/* ungei_d */
     16,	/* ungtr_d */
-    36,	/* ungti_d */
+    44,	/* ungti_d */
     16,	/* ltgtr_d */
-    36,	/* ltgti_d */
+    44,	/* ltgti_d */
     16,	/* ordr_d */
-    36,	/* ordi_d */
+    44,	/* ordi_d */
     16,	/* unordr_d */
-    36,	/* unordi_d */
-    12,	/* truncr_d_i */
+    44,	/* unordi_d */
+    16,	/* truncr_d_i */
     0,	/* truncr_d_l */
-    12,	/* extr_d */
+    20,	/* extr_d */
     4,	/* extr_f_d */
     4,	/* movr_d */
-    20,	/* movi_d */
+    28,	/* movi_d */
     4,	/* ldr_d */
     12,	/* ldi_d */
-    4,	/* ldxr_d */
-    4,	/* ldxi_d */
+    8,	/* ldxr_d */
+    12,	/* ldxi_d */
     4,	/* str_d */
     12,	/* sti_d */
     8,	/* stxr_d */
-    4,	/* stxi_d */
+    12,	/* stxi_d */
     16,	/* bltr_d */
-    36,	/* blti_d */
+    44,	/* blti_d */
     16,	/* bler_d */
-    36,	/* blei_d */
+    44,	/* blei_d */
     16,	/* beqr_d */
-    36,	/* beqi_d */
+    44,	/* beqi_d */
     16,	/* bger_d */
-    36,	/* bgei_d */
+    44,	/* bgei_d */
     16,	/* bgtr_d */
-    36,	/* bgti_d */
+    44,	/* bgti_d */
     16,	/* bner_d */
-    36,	/* bnei_d */
+    44,	/* bnei_d */
     16,	/* bunltr_d */
-    36,	/* bunlti_d */
+    44,	/* bunlti_d */
     16,	/* bunler_d */
-    36,	/* bunlei_d */
+    44,	/* bunlei_d */
     16,	/* buneqr_d */
-    36,	/* buneqi_d */
+    44,	/* buneqi_d */
     16,	/* bunger_d */
-    36,	/* bungei_d */
+    44,	/* bungei_d */
     16,	/* bungtr_d */
-    36,	/* bungti_d */
+    44,	/* bungti_d */
     16,	/* bltgtr_d */
-    36,	/* bltgti_d */
+    44,	/* bltgti_d */
     16,	/* bordr_d */
-    36,	/* bordi_d */
+    44,	/* bordi_d */
     16,	/* bunordr_d */
-    36,	/* bunordi_d */
+    44,	/* bunordi_d */
     0,	/* pushargr_d */
     0,	/* pushargi_d */
     0,	/* retr_d */
@@ -399,4 +402,9 @@
     0,	/* movi_d_ww */
     0,	/* movr_d_w */
     0,	/* movi_d_w */
+    28,	/* bswapr_us */
+    68,	/* bswapr_ui */
+    0,	/* bswapr_ul */
+    88,	/* casr */
+    96,	/* casi */
 #endif /* __WORDSIZE */
