@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017  Free Software Foundation, Inc.
+ * Copyright (C) 2014-2022  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -21,10 +21,11 @@ static char *code_name[] = {
     "data",
     "live",		"align",
     "save",		"load",
+    "skip",
     "#name",		"#note",
     "label",
     "prolog",
-    "ellipsis",
+    "ellipsis",		"va_push",
     "allocai",		"allocar",
     "arg",
     "getarg_c",		"getarg_uc",
@@ -69,6 +70,7 @@ static char *code_name[] = {
     "gtr_u",		"gti_u",
     "ner",		"nei",
     "movr",		"movi",
+    "movnr",		"movzr",
     "extr_c",		"extr_uc",
     "extr_s",		"extr_us",
     "extr_i",		"extr_ui",
@@ -227,4 +229,7 @@ static char *code_name[] = {
     "movr_f_w",		"movi_f_w",
     "movr_d_ww",	"movi_d_ww",
     "movr_d_w",		"movi_d_w",
+    "bswapr_us",
+    "bswapr_ui",		"bswapr_ul",
+    "casr",		"casi",
 };
