@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2017  Free Software Foundation, Inc.
+ * Copyright (C) 2012-2022  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -18,7 +18,24 @@
  */
 
 #define jit_arg_reg_p(i)		((i) >= 0 && (i) < 8)
-#define jit_arg_f_reg_p(i)		((i) >= 0 && (i) < 13)
+#if !_CALL_SYSV
+#  define jit_arg_f_reg_p(i)		((i) >= 0 && (i) < 13)
+#else
+#  define jit_arg_f_reg_p(i)		((i) >= 0 && (i) < 8)
+#  if __WORDSIZE == 32
+#    define va_gp_shift			2
+#  else
+#    define va_gp_shift			3
+#  endif
+#  define va_gp_increment		sizeof(jit_word_t)
+#  define first_gp_argument		r3
+#  define first_gp_offset		offsetof(jit_va_list_t,		\
+						 first_gp_argument)
+#  define va_fp_increment		sizeof(jit_float64_t)
+#  define first_fp_argument		f1
+#  define first_fp_offset		offsetof(jit_va_list_t,		\
+						 first_fp_argument)
+#endif
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #  define C_DISP			0
 #  define S_DISP			0
@@ -34,7 +51,39 @@
 /*
  * Types
  */
+#if _CALL_SYSV
+typedef struct jit_va_list {
+    jit_uint8_t		ngpr;
+    jit_uint8_t		nfpr;
+    jit_uint16_t	_pad;
+#  if __WORDSIZE == 64
+    jit_uint32_t	_pad2;
+#  endif
+    jit_pointer_t	over;
+    jit_pointer_t	save;
+#  if __WORDSIZE == 32
+    jit_word_t		_pad2;
+#  endif
+    jit_word_t		r3;
+    jit_word_t		r4;
+    jit_word_t		r5;
+    jit_word_t		r6;
+    jit_word_t		r7;
+    jit_word_t		r8;
+    jit_word_t		r9;
+    jit_word_t		r10;
+    jit_float64_t	f1;
+    jit_float64_t	f2;
+    jit_float64_t	f3;
+    jit_float64_t	f4;
+    jit_float64_t	f5;
+    jit_float64_t	f6;
+    jit_float64_t	f7;
+    jit_float64_t	f8;
+} jit_va_list_t;
+#else
 typedef jit_pointer_t jit_va_list_t;
+#endif
 
 /*
  * Prototypes
@@ -54,18 +103,11 @@ extern void __clear_cache(void *, void *);
  * Initialization
  */
 jit_register_t		_rvs[] = {
-    { rc(gpr) | 0,			"r0" },
-#if __ppc__
-    { rc(gpr) | 11,			"r11" },
-    { rc(gpr) | 12,			"r12" },
-    { rc(gpr) | 13,			"r13" },
-    { rc(gpr) | 2,			"r2" },
-#else
+    { rc(sav) | 0,			"r0" },
     { rc(sav) | 11,			"r11" },	/* env */
     { rc(sav) | 12,			"r12" },	/* exception */
     { rc(sav) | 13,			"r13" },	/* thread */
     { rc(sav) | 2,			"r2" },		/* toc */
-#endif
     { rc(sav) | rc(gpr) | 14,		"r14" },
     { rc(sav) | rc(gpr) | 15,		"r15" },
     { rc(sav) | rc(gpr) | 16,		"r16" },
@@ -112,11 +154,19 @@ jit_register_t		_rvs[] = {
     { rc(sav) | rc(fpr) | 29,		"f29" },
     { rc(sav) | rc(fpr) | 30,		"f30" },
     { rc(sav) | rc(fpr) | 31,		"f31" },
+#if !_CALL_SYSV
     { rc(arg) | rc(fpr) | 13,		"f13" },
     { rc(arg) | rc(fpr) | 12,		"f12" },
     { rc(arg) | rc(fpr) | 11,		"f11" },
     { rc(arg) | rc(fpr) | 10,		"f10" },
     { rc(arg) | rc(fpr) | 9,		"f9" },
+#else
+    { rc(fpr) | 13,			"f13" },
+    { rc(fpr) | 12,			"f12" },
+    { rc(fpr) | 11,			"f11" },
+    { rc(fpr) | 10,			"f10" },
+    { rc(fpr) | 9,			"f9" },
+#endif
     { rc(arg) | rc(fpr) | 8,		"f8" },
     { rc(arg) | rc(fpr) | 7,		"f7" },
     { rc(arg) | rc(fpr) | 6,		"f6" },
@@ -246,8 +296,7 @@ _jit_retr(jit_state_t *_jit, jit_int32_t u)
     jit_inc_synth_w(retr, u);
     if (JIT_RET != u)
 	jit_movr(JIT_RET, u);
-    else
-	jit_live(JIT_RET);
+    jit_live(JIT_RET);
     jit_ret();
     jit_dec_synth();
 }
@@ -334,24 +383,43 @@ _jit_ellipsis(jit_state_t *_jit)
 	jit_link_prolog();
 	assert(!(_jitc->function->self.call & jit_call_varargs));
 	_jitc->function->self.call |= jit_call_varargs;
-
+#if _CALL_SYSV
+	/* Allocate va_list like object in the stack.
+	 * If applicable, with enough space to save all argument
+	 * registers, and use fixed offsets for them. */
+	_jitc->function->vaoff = jit_allocai(sizeof(jit_va_list_t));
+#endif
 	_jitc->function->vagp = _jitc->function->self.argi;
 	_jitc->function->vafp = _jitc->function->self.argf;
     }
     jit_dec_synth();
 }
 
+void
+_jit_va_push(jit_state_t *_jit, jit_int32_t u)
+{
+    jit_inc_synth_w(va_push, u);
+    jit_pushargr(u);
+    jit_dec_synth();
+}
+
 jit_node_t *
 _jit_arg(jit_state_t *_jit)
 {
     jit_node_t		*node;
     jit_int32_t		 offset;
+    jit_bool_t		 incr = 1;
     assert(_jitc->function);
-    if (jit_arg_reg_p(_jitc->function->self.argi))
+    if (jit_arg_reg_p(_jitc->function->self.argi)) {
 	offset = _jitc->function->self.argi++;
+#if _CALL_SYSV
+	incr = 0;
+#endif
+    }
     else
 	offset = _jitc->function->self.size;
-    _jitc->function->self.size += sizeof(jit_word_t);
+    if (incr)
+	_jitc->function->self.size += sizeof(jit_word_t);
     node = jit_new_node_ww(jit_code_arg, offset,
 			   ++_jitc->function->self.argn);
     jit_link_prolog();
@@ -363,19 +431,27 @@ _jit_arg_f(jit_state_t *_jit)
 {
     jit_node_t		*node;
     jit_int32_t		 offset;
+    jit_bool_t		 incr = 1;
     assert(_jitc->function);
-    if (jit_arg_f_reg_p(_jitc->function->self.argf))
+    if (jit_arg_f_reg_p(_jitc->function->self.argf)) {
 	offset = _jitc->function->self.argf++;
-    else
-	offset = _jitc->function->self.size + F_DISP;
-    if (jit_arg_reg_p(_jitc->function->self.argi)) {
-#if __WORDSIZE == 32
-	_jitc->function->self.argi += 2;
-#else
-	_jitc->function->self.argi++;
+#if _CALL_SYSV
+	incr = 0;
 #endif
     }
-    _jitc->function->self.size += sizeof(jit_word_t);
+    else
+	offset = _jitc->function->self.size + F_DISP;
+#if !_CALL_SYSV
+    if (jit_arg_reg_p(_jitc->function->self.argi)) {
+#  if __WORDSIZE == 32
+	_jitc->function->self.argi += 2;
+#  else
+	_jitc->function->self.argi++;
+#  endif
+    }
+#endif
+    if (incr)
+	_jitc->function->self.size += sizeof(jit_word_t);
     node = jit_new_node_ww(jit_code_arg_f, offset,
 			   ++_jitc->function->self.argn);
     jit_link_prolog();
@@ -387,19 +463,32 @@ _jit_arg_d(jit_state_t *_jit)
 {
     jit_node_t		*node;
     jit_int32_t		 offset;
+    jit_bool_t		 incr = 1;
     assert(_jitc->function);
-    if (jit_arg_f_reg_p(_jitc->function->self.argf))
+    if (jit_arg_f_reg_p(_jitc->function->self.argf)) {
 	offset = _jitc->function->self.argf++;
-    else
-	offset = _jitc->function->self.size;
-    if (jit_arg_reg_p(_jitc->function->self.argi)) {
-#if __WORDSIZE == 32
-	_jitc->function->self.argi += 2;
-#else
-	_jitc->function->self.argi++;
+#if _CALL_SYSV
+	incr = 0;
 #endif
     }
-    _jitc->function->self.size += sizeof(jit_float64_t);
+    else {
+#if _CALL_SYSV
+	if (_jitc->function->self.size & 7)
+	    _jitc->function->self.size += 4;
+#endif
+	offset = _jitc->function->self.size;
+    }
+#if !_CALL_SYSV
+    if (jit_arg_reg_p(_jitc->function->self.argi)) {
+#  if __WORDSIZE == 32
+	_jitc->function->self.argi += 2;
+#  else
+	_jitc->function->self.argi++;
+#  endif
+    }
+#endif
+    if (incr)
+	_jitc->function->self.size += sizeof(jit_float64_t);
     node = jit_new_node_ww(jit_code_arg_d, offset,
 			   ++_jitc->function->self.argn);
     jit_link_prolog();
@@ -611,16 +700,21 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 void
 _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_bool_t		incr = 1;
     assert(_jitc->function);
     jit_inc_synth_w(pushargr, u);
     jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr(JIT_RA0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
+#if _CALL_SYSV
+	incr = 0;
+#endif
     }
     else
 	jit_stxi(_jitc->function->call.size + params_offset, JIT_SP, u);
-    _jitc->function->call.size += sizeof(jit_word_t);
+    if (incr)
+	_jitc->function->call.size += sizeof(jit_word_t);
     jit_dec_synth();
 }
 
@@ -628,12 +722,16 @@ void
 _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 {
     jit_int32_t		 regno;
+    jit_bool_t		 incr = 1;
     assert(_jitc->function);
     jit_inc_synth_w(pushargi, u);
     jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi(JIT_RA0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
+#if _CALL_SYSV
+	incr = 0;
+#endif
     }
     else {
 	regno = jit_get_reg(jit_class_gpr);
@@ -641,148 +739,187 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 	jit_stxi(_jitc->function->call.size + params_offset, JIT_SP, regno);
 	jit_unget_reg(regno);
     }
-    _jitc->function->call.size += sizeof(jit_word_t);
+    if (incr)
+	_jitc->function->call.size += sizeof(jit_word_t);
     jit_dec_synth();
 }
 
 void
 _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_bool_t		incr = 1;
     assert(_jitc->function);
     jit_inc_synth_w(pushargr_f, u);
     jit_link_prepare();
-    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
-	!(_jitc->function->call.call & jit_call_varargs)) {
+    if (jit_arg_f_reg_p(_jitc->function->call.argf)
+#if !_CALL_SYSV
+	&& !(_jitc->function->call.call & jit_call_varargs)
+#endif
+	) {
 	jit_movr_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
+#if !_CALL_SYSV
 	/* in case of excess arguments */
 	if (jit_arg_reg_p(_jitc->function->call.argi)) {
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	    _jitc->function->call.argi += 2;
 	    if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
 		--_jitc->function->call.argi;
-#else
+#  else
 	    _jitc->function->call.argi++;
-#endif
+#  endif
 	}
-    }
-    else if (jit_arg_reg_p(_jitc->function->call.argi
-#if __WORDSIZE == 32
-			  + 1
+#elif _CALL_SYSV
+	incr = 0;
 #endif
+    }
+#if !_CALL_SYSV
+    else if (jit_arg_reg_p(_jitc->function->call.argi
+#  if __WORDSIZE == 32
+			  + 1
+#  endif
 			   )) {
 	/* use reserved 8 bytes area */
 	jit_stxi_d(alloca_offset - 8, JIT_FP, u);
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		 alloca_offset - 8);
 	_jitc->function->call.argi++;
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		 alloca_offset - 4);
 	_jitc->function->call.argi++;
-#endif
+#  endif
     }
+#endif
     else
 	jit_stxi_f(_jitc->function->call.size + params_offset + F_DISP,
 		   JIT_SP, u);
-    _jitc->function->call.size += sizeof(jit_word_t);
+    if (incr)
+	_jitc->function->call.size += sizeof(jit_word_t);
     jit_dec_synth();
 }
 
 void
 _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 {
+    jit_bool_t		 incr = 1;
     jit_int32_t		 regno;
     assert(_jitc->function);
     jit_inc_synth_f(pushargi_f, u);
     jit_link_prepare();
-    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
-	!(_jitc->function->call.call & jit_call_varargs)) {
+    if (jit_arg_f_reg_p(_jitc->function->call.argf)
+#if !_CALL_SYSV
+	&& !(_jitc->function->call.call & jit_call_varargs)
+#endif
+	) {
 	jit_movi_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
+#if !_CALL_SYSV
 	    /* in case of excess arguments */
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	_jitc->function->call.argi += 2;
 	if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
 	    --_jitc->function->call.argi;
-#else
+#  else
 	_jitc->function->call.argi++;
+#  endif
+#elif _CALL_SYSV
+	incr = 0;
 #endif
     }
     else {
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_f(regno, u);
+#if !_CALL_SYSV
 	if (jit_arg_reg_p(_jitc->function->call.argi
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 			  + 1
-#endif
+#  endif
 			  )) {
 	    /* use reserved 8 bytes area */
 	    jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		     alloca_offset - 8);
 	    _jitc->function->call.argi++;
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		     alloca_offset - 4);
 	    _jitc->function->call.argi++;
-#endif
+#  endif
 	}
 	else
+#endif
 	    jit_stxi_f(_jitc->function->call.size + params_offset + F_DISP,
 		       JIT_SP, regno);
 	jit_unget_reg(regno);
     }
-    _jitc->function->call.size += sizeof(jit_word_t);
+    if (incr)
+	_jitc->function->call.size += sizeof(jit_word_t);
     jit_dec_synth();
 }
 
 void
 _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_bool_t		incr = 1;
     assert(_jitc->function);
     jit_inc_synth_w(pushargr_d, u);
     jit_link_prepare();
-    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
-	!(_jitc->function->call.call & jit_call_varargs)) {
+    if (jit_arg_f_reg_p(_jitc->function->call.argf)
+#if !_CALL_SYSV
+	&& !(_jitc->function->call.call & jit_call_varargs)
+#endif
+	) {
 	jit_movr_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
+#if !_CALL_SYSV
 	    /* in case of excess arguments */
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	_jitc->function->call.argi += 2;
 	if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
 	    --_jitc->function->call.argi;
-#else
+#  else
 	_jitc->function->call.argi++;
+#  endif
+#else /* _CALL_SYSV */
+	incr = 0;
 #endif
     }
+#if !_CALL_SYSV
     else if (jit_arg_reg_p(_jitc->function->call.argi
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 			  + 1
-#endif
+#  endif
 			   )) {
 	/* use reserved 8 bytes area */
 	jit_stxi_d(alloca_offset - 8, JIT_FP, u);
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		 alloca_offset - 8);
 	_jitc->function->call.argi++;
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		 alloca_offset - 4);
 	_jitc->function->call.argi++;
-#endif
+#  endif
     }
-    else {
+    else
+#endif /* !_CALL_SYSV */
+    {
+#if _CALL_SYSV
+	if (_jitc->function->call.size & 7)
+	    _jitc->function->call.size += 4;
+#endif
 	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, u);
-#if __WORDSIZE == 32
-	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
-		jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_SP,
-			 _jitc->function->call.size + params_offset);
-		_jitc->function->call.argi++;
-	    }
+#if !_CALL_SYSV && __WORDSIZE == 32
+	if (jit_arg_reg_p(_jitc->function->call.argi)) {
+	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_SP,
+		     _jitc->function->call.size + params_offset);
+	    _jitc->function->call.argi++;
+	}
 #endif
     }
-    _jitc->function->call.size += sizeof(jit_float64_t);
+    if (incr)
+	_jitc->function->call.size += sizeof(jit_float64_t);
     jit_dec_synth();
 }
 
@@ -790,47 +927,62 @@ void
 _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 {
     jit_int32_t		 regno;
+    jit_bool_t		 incr = 1;
     assert(_jitc->function);
     jit_inc_synth_d(pushargi_d, u);
     jit_link_prepare();
-    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
-	!(_jitc->function->call.call & jit_call_varargs)) {
+    if (jit_arg_f_reg_p(_jitc->function->call.argf)
+#if !_CALL_SYSV
+	&& !(_jitc->function->call.call & jit_call_varargs)
+#endif
+	) {
 	jit_movi_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
+#if !_CALL_SYSV
 	/* in case of excess arguments */
 	if (jit_arg_reg_p(_jitc->function->call.argi)) {
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	    _jitc->function->call.argi += 2;
 	    if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
 		--_jitc->function->call.argi;
-#else
+#  else
 	    _jitc->function->call.argi++;
-#endif
+#  endif
 	}
+#else /* _CALL_SYSV */
+	    incr = 0;
+#endif
     }
     else {
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_d(regno, u);
+#if !_CALL_SYSV
 	if (jit_arg_reg_p(_jitc->function->call.argi
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 			  + 1
-#endif
+#  endif
 			  )) {
 	    /* use reserved 8 bytes area */
 	    jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		     alloca_offset - 8);
 	    _jitc->function->call.argi++;
-#if __WORDSIZE == 32
+#  if __WORDSIZE == 32
 	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
 		     alloca_offset - 4);
 	    _jitc->function->call.argi++;
-#endif
+#  endif
 	}
-	else {
+	else
+#endif /* !_CALL_SYSV */
+	{
+#if _CALL_SYSV
+	    if (_jitc->function->call.size & 7)
+		_jitc->function->call.size += 4;
+#endif
 	    jit_stxi_d(_jitc->function->call.size + params_offset,
 		       JIT_SP, regno);
-#if __WORDSIZE == 32
+#if !_CALL_SYSV && __WORDSIZE == 32
 	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
 		jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_SP,
 			 _jitc->function->call.size + params_offset);
@@ -840,7 +992,8 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 	}
 	jit_unget_reg(regno);
     }
-    _jitc->function->call.size += sizeof(jit_float64_t);
+    if (incr)
+	_jitc->function->call.size += sizeof(jit_float64_t);
     jit_dec_synth();
 }
 
@@ -875,6 +1028,11 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
     call = jit_callr(r0);
     call->v.w = _jitc->function->call.argi;
     call->w.w = _jitc->function->call.argf;
+#if _CALL_SYSV
+    /* If passing float arguments in registers */
+    if ((_jitc->function->call.call & jit_call_varargs) && call->w.w)
+	call->flag |= jit_flag_varargs;
+#endif
     _jitc->function->call.argi = _jitc->function->call.argf = 0;
     _jitc->prepare = 0;
     jit_dec_synth();
@@ -891,6 +1049,10 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
     node = jit_calli(i0);
     node->v.w = _jitc->function->call.argi;
     node->w.w = _jitc->function->call.argf;
+#if _CALL_SYSV
+    if ((_jitc->function->call.call & jit_call_varargs) && node->w.w)
+	node->flag |= jit_flag_varargs;
+#endif
     _jitc->function->call.argi = _jitc->function->call.argf = 0;
     _jitc->prepare = 0;
     jit_dec_synth();
@@ -986,14 +1148,17 @@ _emit_code(jit_state_t *_jit)
     jit_word_t		 word;
     jit_int32_t		 value;
     jit_int32_t		 offset;
+    jit_bool_t       no_flag = 0;	/* Set if previous instruction is
+					 * *not* a jump target. */
     struct {
 	jit_node_t	*node;
 	jit_word_t	 word;
+	jit_function_t	 func;
 #if DEVEL_DISASSEMBLER
 	jit_word_t	 prevw;
 #endif
 	jit_word_t	 patch_offset;
-#if __powerpc__
+#if _CALL_AIXDESC
 	jit_word_t	 prolog_offset;
 #endif
     } undo;
@@ -1012,7 +1177,7 @@ _emit_code(jit_state_t *_jit)
 #if DEVEL_DISASSEMBLER
     prevw = _jit->pc.w;
 #endif
-#if __powerpc__ && !ABI_ELFv2
+#if _CALL_AIXDESC
     undo.prolog_offset = 0;
     for (node = _jitc->head; node; node = node->next)
 	if (node->code != jit_code_label &&
@@ -1124,11 +1289,13 @@ _emit_code(jit_state_t *_jit)
 	jit_regarg_set(node, value);
 	switch (node->code) {
 	    case jit_code_align:
-		assert(!(node->u.w & (node->u.w - 1)) &&
-		       node->u.w <= sizeof(jit_word_t));
-		if (node->u.w == sizeof(jit_word_t) &&
-		    (word = _jit->pc.w & (sizeof(jit_word_t) - 1)))
-		    nop(sizeof(jit_word_t) - word);
+		/* Must align to a power of two */
+		assert(!(node->u.w & (node->u.w - 1)));
+		if ((word = _jit->pc.w & (node->u.w - 1)))
+		    nop(node->u.w - word);
+		break;
+	    case jit_code_skip:
+	        nop((node->u.w + 3) & ~3);
 		break;
 	    case jit_code_note:		case jit_code_name:
 		node->u.w = _jit->pc.w;
@@ -1193,9 +1360,28 @@ _emit_code(jit_state_t *_jit)
 		case_rr(hton, _ui);
 #  if __WORDSIZE == 64
 		case_rr(hton, _ul);
+#  endif
+	    case jit_code_bswapr_us:
+		bswapr_us_lh(rn(node->u.w), rn(node->v.w), no_flag);
+		break;
+	    case jit_code_bswapr_ui:
+		bswapr_ui_lw(rn(node->u.w), rn(node->v.w), no_flag);
+		break;
+#  if __WORDSIZE == 64
+		case_rr(bswap, _ul);
 #  endif
 		case_rr(neg,);
 		case_rr(com,);
+	    case jit_code_casr:
+		casr(rn(node->u.w), rn(node->v.w),
+		     rn(node->w.q.l), rn(node->w.q.h));
+		break;
+	    case jit_code_casi:
+		casi(rn(node->u.w), node->v.w,
+		     rn(node->w.q.l), rn(node->w.q.h));
+		break;
+		case_rrr(movn,);
+		case_rrr(movz,);
 		case_rr(mov,);
 	    case jit_code_movi:
 		if (node->flag & jit_flag_node) {
@@ -1499,7 +1685,7 @@ _emit_code(jit_state_t *_jit)
 		break;
 	    case jit_code_jmpi:
 		if (node->flag & jit_flag_node) {
-#if __powerpc__ && !ABI_ELFv2
+#if _CALL_AIXDESC
 		    if (_jit->pc.uc == _jit->code.ptr + sizeof(void*) * 3)
 			_jitc->jump = 1;
 #endif
@@ -1509,42 +1695,68 @@ _emit_code(jit_state_t *_jit)
 		    if (temp->flag & jit_flag_patch)
 			jmpi(temp->u.w);
 		    else {
-			word = jmpi(_jit->pc.w);
+			word = _jit->code.length -
+			    (_jit->pc.uc - _jit->code.ptr);
+			if (can_sign_extend_jump_p(word))
+			    word = jmpi(_jit->pc.w);
+			else
+			    word = jmpi_p(_jit->pc.w);
 			patch(word, node);
 		    }
 		}
 		else
-		    (void)jmpi_p(node->u.w);
+		    jmpi(node->u.w);
 		break;
 	    case jit_code_callr:
-		callr(rn(node->u.w));
+#if _CALL_SYSV
+#  define xcallr(u, v)		callr(u, v)
+#  define xcalli_p(u, v)	calli_p(u, v)
+#  define xcalli(u, v)		calli(u, v)
+#else
+#  define xcallr(u, v)		callr(u)
+#  define xcalli_p(u, v)	calli_p(u)
+#  define xcalli(u, v)		calli(u)
+#endif
+		xcallr(rn(node->u.w), !!(node->flag & jit_flag_varargs));
 		break;
 	    case jit_code_calli:
+		value = !!(node->flag & jit_flag_varargs);
 		if (node->flag & jit_flag_node) {
 		    temp = node->u.n;
 		    assert(temp->code == jit_code_label ||
 			   temp->code == jit_code_epilog);
-		    word = calli_p(temp->u.w);
-		    if (!(temp->flag & jit_flag_patch))
+		    if (temp->flag & jit_flag_patch)
+			xcalli(temp->u.w, value);
+		    else {
+			word = _jit->code.length -
+			    (_jit->pc.uc - _jit->code.ptr);
+#if _CALL_SYSV
+			if (can_sign_extend_jump_p(word + value * 4))
+			    word = xcalli(_jit->pc.w, value);
+			else
+#endif
+			    word = xcalli_p(_jit->pc.w, value);
 			patch(word, node);
+		    }
 		}
 		else
-		    calli(node->u.w);
+		    xcalli(node->u.w, value);
 		break;
 	    case jit_code_prolog:
 		_jitc->function = _jitc->functions.ptr + node->w.w;
 		undo.node = node;
 		undo.word = _jit->pc.w;
+		memcpy(&undo.func, _jitc->function, sizeof(undo.func));
 #if DEVEL_DISASSEMBLER
 		undo.prevw = prevw;
 #endif
 		undo.patch_offset = _jitc->patches.offset;
-#if __powerpc__ && !ABI_ELFv2
+#if _CALL_AIXDESC
 		undo.prolog_offset = _jitc->prolog.offset;
 #endif
 	    restart_function:
 		_jitc->again = 0;
-#if __powerpc__ && !ABI_ELFv2
+#if _CALL_AIXDESC
 		if (_jitc->jump && !_jitc->function->assume_frame) {
 		    /* remember prolog to hide offset adjustment for a jump
 		     * to the start of a function, what is expected to be
@@ -1578,11 +1790,18 @@ _emit_code(jit_state_t *_jit)
 		    temp->flag &= ~jit_flag_patch;
 		    node = undo.node;
 		    _jit->pc.w = undo.word;
+		    /* undo.func.self.aoff and undo.func.regset should not
+		     * be undone, as they will be further updated, and are
+		     * the reason of the undo. */
+		    undo.func.self.aoff = _jitc->function->frame +
+			_jitc->function->self.aoff;
+		    jit_regset_set(&undo.func.regset, &_jitc->function->regset);
+		    memcpy(_jitc->function, &undo.func, sizeof(undo.func));
 #if DEVEL_DISASSEMBLER
 		    prevw = undo.prevw;
 #endif
 		    _jitc->patches.offset = undo.patch_offset;
-#if __powerpc__ && !ABI_ELFv2
+#if _CALL_AIXDESC
 		    _jitc->prolog.offset = undo.prolog_offset;
 #endif
 		    goto restart_function;
@@ -1604,6 +1823,7 @@ _emit_code(jit_state_t *_jit)
 		break;
 	    case jit_code_live:
 	    case jit_code_arg:			case jit_code_ellipsis:
+	    case jit_code_va_push:
 	    case jit_code_allocai:		case jit_code_allocar:
 	    case jit_code_arg_f:		case jit_code_arg_d:
 	    case jit_code_va_end:
@@ -1641,6 +1861,8 @@ _emit_code(jit_state_t *_jit)
 	assert(_jitc->regarg == 0 && _jitc->synth == 0);
 	/* update register live state */
 	jit_reglive(node);
+
+        no_flag = !(node->flag & jit_flag_patch);
     }
 #undef case_brf
 #undef case_brw
