@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2017  Free Software Foundation, Inc.
+ * Copyright (C) 2013-2022  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -28,7 +28,7 @@
  */
 static jit_int16_t	_szs[jit_code_last_code] = {
 #if GET_JIT_SIZE
-#  define JIT_INSTR_MAX		256
+#  define JIT_INSTR_MAX		512
 #else
 #  if defined(__i386__) || defined(__x86_64__)
 #    include "jit_x86-sz.c"
@@ -36,7 +36,7 @@ static jit_int16_t	_szs[jit_code_last_code] = {
 #    include "jit_mips-sz.c"
 #  elif defined(__arm__)
 #    include "jit_arm-sz.c"
-#  elif defined(__ppc__) || defined(__powerpc__)
+#  elif defined(__powerpc__)
 #    include "jit_ppc-sz.c"
 #  elif defined(__sparc__)
 #    include "jit_sparc-sz.c"
@@ -50,6 +50,10 @@ static jit_int16_t	_szs[jit_code_last_code] = {
 #    include "jit_s390-sz.c"
 #  elif defined(__alpha__)
 #    include "jit_alpha-sz.c"
+#  elif defined(__riscv)
+#    include "jit_riscv-sz.c"
+#  elif defined(__loongarch__)
+#    include "jit_loongarch-sz.c"
 #  endif
 #endif
 };
@@ -99,11 +103,40 @@ _jit_get_size(jit_state_t *_jit)
 {
     jit_word_t		 size;
     jit_node_t		*node;
+#  if __riscv && __WORDSIZE == 64
+    jit_word_t		 extra = 0;
+#  endif
 
-    for (size = JIT_INSTR_MAX, node = _jitc->head; node; node = node->next)
-	size += _szs[node->code];
+    for (size = JIT_INSTR_MAX, node = _jitc->head; node; node = node->next) {
+#  if __riscv && __WORDSIZE == 64
+	/* Get estimative of extra memory for constants at end of code. */
+	switch (node->code) {
+	    case jit_code_movi:
+	    case jit_code_movi_f:
+	    case jit_code_movi_d:
+	    case jit_code_jmpi:
+	    case jit_code_calli:
+		extra += sizeof(jit_word_t);
+	    default:
+		break;
+	}
+#  endif
+	switch (node->code) {
+	    /* The instructions are special because they can be arbitrarily long.  */
+	    case jit_code_align:
+	    case jit_code_skip:
+	        size += node->u.w;
+	        break;
+	    default:
+	        size += _szs[node->code];
+	}
+    }
+#  if __riscv && __WORDSIZE == 64
+    /* Heuristically only 20% of constants are unique. */
+    size += extra / 5;
+#  endif
 
-    return ((size + 4095) & -4096);
+    return size;
 }
 #endif
 
