@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2019  Free Software Foundation, Inc.
+ * Copyright (C) 2012-2022  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -30,7 +30,7 @@
 #  define va_gp_increment		4
 #  define va_fp_increment		8
 #else
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
 #    define jit_arg_reg_p(i)		((i) >= 0 && (i) < 4)
 #    define jit_arg_f_reg_p(i)		jit_arg_reg_p(i)
 #    define stack_framesize		152
@@ -62,7 +62,7 @@
 /*
  * Types
  */
-#if __X32 || __CYGWIN__
+#if __X32 || __CYGWIN__ || _WIN32
 typedef jit_pointer_t jit_va_list_t;
 #else
 typedef struct jit_va_list {
@@ -147,7 +147,7 @@ jit_register_t		_rvs[] = {
     { rc(fpr) | 6,			"st(6)" },
     { rc(fpr) | 7,			"st(7)" },
 #else
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     { rc(gpr) | rc(rg8) | 0,		"%rax" },
     { rc(gpr) | rc(rg8) | rc(rg8) | 10,	"%r10" },
     { rc(gpr) | rc(rg8) | rc(rg8) | 11,	"%r11" },
@@ -597,7 +597,7 @@ _jit_ellipsis(jit_state_t *_jit)
 	assert(!(_jitc->function->self.call & jit_call_varargs));
 	_jitc->function->self.call |= jit_call_varargs;
 
-#if __X64 && !__CYGWIN__
+#if __X64 && !(__CYGWIN__ || _WIN32)
 	/* Allocate va_list like object in the stack.
 	 * If applicable, with enough space to save all argument
 	 * registers, and use fixed offsets for them. */
@@ -638,7 +638,7 @@ _jit_arg(jit_state_t *_jit)
 #if __X64
     if (jit_arg_reg_p(_jitc->function->self.argi)) {
 	offset = _jitc->function->self.argi++;
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
 	_jitc->function->self.size += sizeof(jit_word_t);
 #  endif
     }
@@ -662,7 +662,7 @@ _jit_arg_f(jit_state_t *_jit)
     assert(_jitc->function);
     assert(!(_jitc->function->self.call & jit_call_varargs));
 #if __X64
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     if (jit_arg_reg_p(_jitc->function->self.argi)) {
 	offset = _jitc->function->self.argi++;
 	_jitc->function->self.size += sizeof(jit_word_t);
@@ -691,7 +691,7 @@ _jit_arg_d(jit_state_t *_jit)
     assert(_jitc->function);
     assert(!(_jitc->function->self.call & jit_call_varargs));
 #if __X64
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     if (jit_arg_reg_p(_jitc->function->self.argi)) {
 	offset = _jitc->function->self.argi++;
 	_jitc->function->self.size += sizeof(jit_word_t);
@@ -953,7 +953,7 @@ _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr(JIT_RA0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
 	if (_jitc->function->call.call & jit_call_varargs)
 	    jit_stxi(_jitc->function->call.size, _RSP, u);
 	_jitc->function->call.size += sizeof(jit_word_t);
@@ -978,7 +978,7 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 #if __X64
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi(JIT_RA0 - _jitc->function->call.argi, u);
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
 	if (_jitc->function->call.call & jit_call_varargs)
 	    jit_stxi(_jitc->function->call.size, _RSP,
 		     JIT_RA0 - _jitc->function->call.argi);
@@ -1005,7 +1005,7 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
     jit_inc_synth_w(pushargr_f, u);
     jit_link_prepare();
 #if __X64
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr_f(_XMM0 - _jitc->function->call.argi, u);
 	if (_jitc->function->call.call & jit_call_varargs) {
@@ -1040,7 +1040,7 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
     jit_inc_synth_f(pushargi_f, u);
     jit_link_prepare();
 #if __X64
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi_f(_XMM0 - _jitc->function->call.argi, u);
 	if (_jitc->function->call.call & jit_call_varargs) {
@@ -1077,7 +1077,7 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
     jit_inc_synth_w(pushargr_d, u);
     jit_link_prepare();
 #if __X64
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr_d(_XMM0 - _jitc->function->call.argi, u);
 	if (_jitc->function->call.call & jit_call_varargs) {
@@ -1112,7 +1112,7 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
     jit_inc_synth_d(pushargi_d, u);
     jit_link_prepare();
 #if __X64
-#  if __CYGWIN__
+#  if __CYGWIN__ || _WIN32
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi_d(_XMM0 - _jitc->function->call.argi, u);
 	if (_jitc->function->call.call & jit_call_varargs) {
@@ -1176,7 +1176,7 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
 #if __X64
-#  if !__CYGWIN__
+#  if !(__CYGWIN__ || _WIN32)
     if (_jitc->function->call.call & jit_call_varargs) {
 	if (jit_regno(reg) == _RAX) {
 	    reg = jit_get_reg(jit_class_gpr);
@@ -1212,23 +1212,19 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
 #if __X64
-    /* FIXME preventing %rax allocation is good enough, but for consistency
-     * it should automatically detect %rax is dead, in case it has run out
-     * registers, and not save/restore it, what would be wrong if using the
-     * the return value, otherwise, just a needless noop */
-    /* >> prevent %rax from being allocated as the function pointer */
-    jit_regset_setbit(&_jitc->regarg, _RAX);
-    reg = jit_get_reg(jit_class_gpr);
-    node = jit_movi(reg, (jit_word_t)i0);
-    jit_finishr(reg);
-    jit_unget_reg(reg);
-    /* << prevent %rax from being allocated as the function pointer */
-    jit_regset_clrbit(&_jitc->regarg, _RAX);
-#else
+#  if !(__CYGWIN__ || _WIN32)
+    if (_jitc->function->call.call & jit_call_varargs) {
+	if (_jitc->function->call.argf)
+	    jit_movi(_RAX, _jitc->function->call.argf);
+	else
+	    jit_movi(_RAX, 0);
+	jit_live(_RAX);
+    }
+#  endif
+#endif
     node = jit_calli(i0);
     node->v.w = _jitc->function->call.argi;
     node->w.w = _jitc->function->call.argf;
-#endif
     _jitc->function->call.argi = _jitc->function->call.argf =
 	_jitc->function->call.size = 0;
     _jitc->prepare = 0;
@@ -1333,6 +1329,7 @@ _emit_code(jit_state_t *_jit)
     struct {
 	jit_node_t	*node;
 	jit_word_t	 word;
+	jit_function_t	 func;
 #if DEVEL_DISASSEMBLER
 	jit_word_t	 prevw;
 #endif
@@ -1593,12 +1590,15 @@ _emit_code(jit_state_t *_jit)
 	jit_regarg_set(node, value);
 	switch (node->code) {
 	    case jit_code_align:
-		assert(!(node->u.w & (node->u.w - 1)) &&
-		       node->u.w <= sizeof(jit_word_t));
+		/* Must align to a power of two */
+		assert(!(node->u.w & (node->u.w - 1)));
 		if ((word = _jit->pc.w & (node->u.w - 1)))
 		    nop(node->u.w - word);
 		break;
-	    case jit_code_note:		case jit_code_name:
+            case jit_code_skip:
+                nop(node->u.w);
+                break;
+            case jit_code_note:		case jit_code_name:
 		node->u.w = _jit->pc.w;
 		break;
 	    case jit_code_label:
@@ -1674,6 +1674,16 @@ _emit_code(jit_state_t *_jit)
 		case_rrw(gt, _u);
 		case_rrr(ne,);
 		case_rrw(ne,);
+	    case jit_code_casr:
+		casr(rn(node->u.w), rn(node->v.w),
+		     rn(node->w.q.l), rn(node->w.q.h));
+		break;
+	    case jit_code_casi:
+		casi(rn(node->u.w), node->v.w,
+		     rn(node->w.q.l), rn(node->w.q.h));
+		break;
+		case_rrr(movn,);
+		case_rrr(movz,);
 		case_rr(mov,);
 	    case jit_code_movi:
 		if (node->flag & jit_flag_node) {
@@ -1696,6 +1706,11 @@ _emit_code(jit_state_t *_jit)
 		case_rr(hton, _ui);
 #if __X64 && !__X64_32
 		case_rr(hton, _ul);
+#endif
+		case_rr(bswap, _us);
+		case_rr(bswap, _ui);
+#if __X64 && !__X64_32
+		case_rr(bswap, _ul);
 #endif
 		case_rr(ext, _c);
 		case_rr(ext, _uc);
@@ -2012,7 +2027,14 @@ _emit_code(jit_state_t *_jit)
 		    if (temp->flag & jit_flag_patch)
 			jmpi(temp->u.w);
 		    else {
-			word = jmpi(_jit->pc.w);
+#if __X64
+			word = _jit->code.length -
+			    (_jit->pc.uc - _jit->code.ptr);
+			if ((jit_int32_t)word == word)
+			    word = jmpi(_jit->pc.w);
+			else
+#endif
+			    word = jmpi_p(_jit->pc.w);
 			patch(word, node);
 		    }
 		}
@@ -2027,9 +2049,19 @@ _emit_code(jit_state_t *_jit)
 		    temp = node->u.n;
 		    assert(temp->code == jit_code_label ||
 			   temp->code == jit_code_epilog);
-		    word = calli(temp->u.w);
-		    if (!(temp->flag & jit_flag_patch))
+		    if (temp->flag & jit_flag_patch)
+			calli(temp->u.w);
+		    else {
+#if __X64
+			word = _jit->code.length -
+			    (_jit->pc.uc - _jit->code.ptr);
+			if ((jit_int32_t)word == word)
+			    word = calli(_jit->pc.w);
+			else
+#endif
+			    word = calli_p(_jit->pc.w);
 			patch(word, node);
+		    }
 		}
 		else
 		    calli(node->u.w);
@@ -2038,6 +2070,7 @@ _emit_code(jit_state_t *_jit)
 		_jitc->function = _jitc->functions.ptr + node->w.w;
 		undo.node = node;
 		undo.word = _jit->pc.w;
+		memcpy(&undo.func, _jitc->function, sizeof(undo.func));
 #if DEVEL_DISASSEMBLER
 		undo.prevw = prevw;
 #endif
@@ -2058,6 +2091,13 @@ _emit_code(jit_state_t *_jit)
 		    temp->flag &= ~jit_flag_patch;
 		    node = undo.node;
 		    _jit->pc.w = undo.word;
+		    /* undo.func.self.aoff and undo.func.regset should not
+		     * be undone, as they will be further updated, and are
+		     * the reason of the undo. */
+		    undo.func.self.aoff = _jitc->function->frame +
+			_jitc->function->self.aoff;
+		    jit_regset_set(&undo.func.regset, &_jitc->function->regset);
+		    memcpy(_jitc->function, &undo.func, sizeof(undo.func));
 #if DEVEL_DISASSEMBLER
 		    prevw = undo.prevw;
 #endif
@@ -2164,7 +2204,7 @@ _emit_code(jit_state_t *_jit)
     for (offset = 0; offset < _jitc->patches.offset; offset++) {
 	node = _jitc->patches.ptr[offset].node;
 	word = node->code == jit_code_movi ? node->v.n->u.w : node->u.n->u.w;
-	patch_at(node, _jitc->patches.ptr[offset].inst, word);
+	patch_at(_jitc->patches.ptr[offset].inst, word);
     }
 
     jit_flush(_jit->code.ptr, _jit->pc.uc);
